@@ -16,14 +16,14 @@
  */
 #define NO_COREXT
 
-static char sccsid[] = "@(#) tkText.c 1.80 95/07/20 09:42:42";
+static char sccsid[] = "@(#) tkText.c 1.82 95/08/21 11:46:27";
 
 #include "default.h"
 #include "tkPort.h"
 #include "tkInt.h"
 #include "tkText.h"
 
-static char rcsid[] = "$Id: tkText.c,v 1.19 1995/09/09 06:44:23 ilya Exp ilya $";
+static char rcsid[] = "$Id: tkText.c,v 1.24 1996/03/20 21:27:06 ilya Exp $";
 
 /*
  * Information used to parse text configuration options:
@@ -290,6 +290,8 @@ Tk_TextCmd(clientData, interp, argc, args)
     textPtr->bindingTable = NULL;
     textPtr->currentMarkPtr = NULL;
     textPtr->pickEvent.type = LeaveNotify;
+    textPtr->pickEvent.xcrossing.x = 0;
+    textPtr->pickEvent.xcrossing.y = 0;
     textPtr->numCurTags = 0;
     textPtr->curTagArrayPtr = NULL;
     textPtr->takeFocus = NULL;
@@ -534,13 +536,13 @@ TextWidgetCmd(clientData, interp, argc, args)
 	offset = index1.charIndex; 
 	Tcl_DStringInit(&dstring);
 	while (linePtr != index2.linePtr) {
-	  TkBTreeLineTextAppend(linePtr, offset, -1, 
+	  TkBTreeLineTextAppend(textPtr, linePtr, offset, -1, 
 				(void*)&dstring, 
 				textPtr->getFlags | TKT_GET_DSTRING, NULL);
 	  offset = 0;
 	  linePtr = TkBTreeNextLine(linePtr);
 	}
-	TkBTreeLineTextAppend(linePtr, offset, index2.charIndex,
+	TkBTreeLineTextAppend(textPtr, linePtr, offset, index2.charIndex,
 			      (void*)&dstring, 
 			      textPtr->getFlags | TKT_GET_DSTRING, NULL);
 	Tcl_AppendResult(interp, dstring.string,          NULL);
@@ -1332,7 +1334,7 @@ TextFetchSelection(clientData, offset, buffer, maxBytes)
 	char *s;
 
 	Tcl_DStringInit(&dString);
-	TkBTreeLineTextAppend(linePtr, textPtr->selIndex.charIndex,
+	TkBTreeLineTextAppend(textPtr, linePtr, textPtr->selIndex.charIndex,
 			      textPtr->selIndex.charIndex + 1, 
 			      (void*)&dString, 
 			      textPtr->getFlags | TKT_GET_DSTRING,
@@ -1381,7 +1383,7 @@ TextFetchSelection(clientData, offset, buffer, maxBytes)
 	    if (linePtr == search.extIndex.linePtr) {
 	      endOff = search.extIndex.charIndex;
 	    }
-	    pos = TkBTreeLineTextAppend(linePtr, startOff, endOff,
+	    pos = TkBTreeLineTextAppend(textPtr, linePtr, startOff, endOff,
 					(void *)buffer, 
 					textPtr->getFlags | TKT_GET_BUFFER,
 					&maxBytes);
@@ -1405,7 +1407,7 @@ TextFetchSelection(clientData, offset, buffer, maxBytes)
 		char *s;
 
 		Tcl_DStringInit(&dString);
-		TkBTreeLineTextAppend(linePtr, pos, pos + 1, 
+		TkBTreeLineTextAppend(textPtr, linePtr, pos, pos + 1, 
 				      (void*)&dString, 
 				      textPtr->getFlags | TKT_GET_DSTRING,
 				      NULL);
@@ -1552,7 +1554,10 @@ TextSearchCmd(textPtr, interp, argc, args)
     Arg varName;
     char buffer[20];
     TkTextIndex index, stopIndex;
-    Tcl_DString line, patDString;
+    Tcl_DString line;
+#if 0
+    Tcl_DString patDString;
+#endif
     TkTextSegment *segPtr;
     TkTextLine *linePtr;
     Tcl_RegExp regExp = NULL;		/* Initialization needed only to
@@ -1580,24 +1585,24 @@ TextSearchCmd(textPtr, interp, argc, args)
 	    return TCL_ERROR;
 	}
 	c = arg[1];
-	if ((c == 'b') && (strncmp(LangString(args[i]), "-backwards", length) == 0)) {
+	if ((c == 'b') &&  LangCmpOpt("-backwards",LangString(args[i]),length) == 0 ) {
 	    backwards = 1;
-	} else if ((c == 'c') && (strncmp(LangString(args[i]), "-count", length) == 0)) {
+	} else if ((c == 'c') &&  LangCmpOpt("-count",LangString(args[i]),length) == 0 ) {
 	    if (i >= (argc-1)) {
 		Tcl_SetResult(interp, "no value given for \"-count\" option",TCL_STATIC);
 		return TCL_ERROR;
 	    }
 	    i++;
 	    varName = args[i];
-	} else if ((c == 'e') && (strncmp(LangString(args[i]), "-exact", length) == 0)) {
+	} else if ((c == 'e') &&  LangCmpOpt("-exact",LangString(args[i]),length) == 0 ) {
 	    exact = 1;
-	} else if ((c == 'f') && (strncmp(LangString(args[i]), "-forwards", length) == 0)) {
+	} else if ((c == 'f') &&  LangCmpOpt("-forwards",LangString(args[i]),length) == 0 ) {
 	    backwards = 0;
-	} else if ((c == 'n') && (strncmp(LangString(args[i]), "-nocase", length) == 0)) {
+	} else if ((c == 'n') &&  LangCmpOpt("-nocase",LangString(args[i]),length) == 0 ) {
 	    noCase = TKT_GET_NOCASE;
-	} else if ((c == 'r') && (strncmp(LangString(args[i]), "-regexp", length) == 0)) {
+	} else if ((c == 'r') &&  LangCmpOpt("-regexp",LangString(args[i]),length) == 0 ) {
 	    exact = 0;
-	} else if ((c == '-') && (strncmp(LangString(args[i]), "--", length) == 0)) {
+	} else if ((c == '-') &&  LangCmpOpt("--",LangString(args[i]),length) == 0 ) {
 	    i++;
 	    break;
 	} else {
@@ -1613,6 +1618,7 @@ TextSearchCmd(textPtr, interp, argc, args)
     }
     pattern = LangString(args[i]);
 
+#if 0
     /*
      * Convert the pattern to lower-case if we're supposed to ignore case.
      */
@@ -1627,6 +1633,7 @@ TextSearchCmd(textPtr, interp, argc, args)
 	    }
 	}
     }
+#endif
 
     if (TkTextGetIndex(interp, textPtr, LangString(args[i+1]), &index) != TCL_OK) {
 	return TCL_ERROR;
@@ -1635,8 +1642,14 @@ TextSearchCmd(textPtr, interp, argc, args)
     startingLine = TkBTreeLineIndex(index.linePtr);
     startingChar = index.charIndex;
     if (startingLine >= numLines) {
-	startingLine = 0;
-	startingChar = 0;
+	if (backwards) {
+	    startingLine = TkBTreeNumLines(textPtr->tree) - 1;
+	    startingChar = TkBTreeCharsInLine(TkBTreeFindLine(textPtr->tree,
+		    startingLine));
+	} else {
+	    startingLine = 0;
+	    startingChar = 0;
+	}
     }
     if (argsLeft == 1) {
 	if (TkTextGetIndex(interp, textPtr, LangString(args[i+2]), &stopIndex) != TCL_OK) {
@@ -1662,7 +1675,7 @@ TextSearchCmd(textPtr, interp, argc, args)
     if (exact) {
 	patLength = strlen(pattern);
     } else {
-	regExp = Tcl_RegExpCompile(interp, pattern);
+	regExp = Lang_RegExpCompile(interp, pattern, noCase);
 	if (regExp == NULL) {
 	    return TCL_ERROR;
 	}
@@ -1695,11 +1708,11 @@ TextSearchCmd(textPtr, interp, argc, args)
 
 	if (lineNum == startingLine) {
 	    passes++;
-	    TkBTreeLineTextAppend(linePtr, 0, startingChar, (void*)&line, 
+	    TkBTreeLineTextAppend(textPtr, linePtr, 0, startingChar, (void*)&line, 
 				  textPtr->getFlags | noCase | TKT_GET_DSTRING,
 				  NULL);
 	    firstChar = Tcl_DStringLength(&line);
-	    TkBTreeLineTextAppend(linePtr, startingChar, -1, (void*)&line, 
+	    TkBTreeLineTextAppend(textPtr, linePtr, startingChar, -1, (void*)&line, 
 				  textPtr->getFlags | noCase | TKT_GET_DSTRING,
 				  NULL);
 	    if ((passes == 1) ^ backwards) {
@@ -1709,7 +1722,7 @@ TextSearchCmd(textPtr, interp, argc, args)
 	      firstChar = 0;
 	    }
 	} else {
-	    TkBTreeLineTextAppend(linePtr, 0, -1, (void*)&line, 
+	    TkBTreeLineTextAppend(textPtr, linePtr, 0, -1, (void*)&line, 
 				  textPtr->getFlags | noCase | TKT_GET_DSTRING,
 				  NULL);
 	    firstChar = 0;
@@ -1720,6 +1733,20 @@ TextSearchCmd(textPtr, interp, argc, args)
 	}
 	startOfLine = Tcl_DStringValue(&line);
 
+	/*
+	 * If we're ignoring case, convert the line to lower case.
+	 */
+
+#if 0
+	if (noCase) {
+	    for (p = Tcl_DStringValue(&line); *p != 0; p++) {
+		if (isupper(UCHAR(*p))) {
+		    *p = tolower(UCHAR(*p));
+		}
+	    }
+	}
+#endif
+
 	/*
 	 * Check for matches within the current line.  If so, and if we're
 	 * searching backwards, repeat the search to find the last match
@@ -1740,7 +1767,7 @@ TextSearchCmd(textPtr, interp, argc, args)
 		char *start, *end;
 		int match;
 
-		match = Tcl_RegExpExec(interp, regExp,
+		match = Lang_RegExpExec(interp, regExp,
 			startOfLine + firstChar, startOfLine);
 		if (match < 0) {
 		    code = TCL_ERROR;
@@ -1775,12 +1802,12 @@ TextSearchCmd(textPtr, interp, argc, args)
 	     */
 
 	    int offset = matchChar;
-	    int posStart = TkBTreeLineTextAppend(linePtr, 0, -1, NULL,
+	    int posStart = TkBTreeLineTextAppend(textPtr, linePtr, 0, -1, NULL,
 						 textPtr->getFlags | TKT_GET_OFFSET,
 						 &offset);
 
 	    offset += matchLength;
-	    matchLength = TkBTreeLineTextAppend(linePtr, posStart, -1, NULL,
+	    matchLength = TkBTreeLineTextAppend(textPtr, linePtr, posStart, -1, NULL,
 						textPtr->getFlags | TKT_GET_OFFSET,
 						&offset) 
 	      - posStart;
@@ -1844,9 +1871,11 @@ TextSearchCmd(textPtr, interp, argc, args)
     }
     done:
     Tcl_DStringFree(&line);
+#if 0
     if (noCase) {
 	Tcl_DStringFree(&patDString);
     }
+#endif
     if (regExp) {
 	Lang_FreeRegExp(regExp);
     }
@@ -2223,7 +2252,10 @@ TkTextBlockCmd(textPtr, interp, argc, args)
 	  textPtr->dlinePtrs = NULL;
 	  TkTextRelayoutWindow(textPtr);
 	}
-    } else if ((c == 'i') && (strncmp(LangString(args[2]), "insert", length) == 0)) {
+    } else if ((c == 'i') && (strncmp(LangString(args[2]), "insert", length) ==
+			      0)) {
+        int lineIndex, numLines;
+
 	if ((argc < 5) || (argc > 6)) {
 	    Tcl_AppendResult(interp, "wrong # args: should be \"",
 		    LangString(args[0]), " block insert blockName position ?position?",
@@ -2236,8 +2268,19 @@ TkTextBlockCmd(textPtr, interp, argc, args)
 		!= TCL_OK)) {
 	  return TCL_ERROR;
 	}
+	lineIndex = TkBTreeLineIndex(index1.linePtr);
+	if (lineIndex == (numLines = TkBTreeNumLines(textPtr->tree))) {
+	    lineIndex--;
+	    TkTextMakeIndex(textPtr->tree, lineIndex, 1000000, &index1);
+	}
 	if (argc < 6) {
-	  index2 = index1;
+	    index2 = index1;
+	} else {
+	        lineIndex = TkBTreeLineIndex(index2.linePtr);
+		if (lineIndex == numLines) {
+		    lineIndex--;
+		    TkTextMakeIndex(textPtr->tree, lineIndex, 1000000, &index2);
+		}
 	}
 
 	hPtr = Tcl_FindHashEntry(&textPtr->blockTable, LangString(args[3]));
@@ -2273,6 +2316,43 @@ TkTextBlockCmd(textPtr, interp, argc, args)
 	    Tcl_AppendElement(interp,
 		    Tcl_GetHashKey(&textPtr->blockTable, hPtr));
 	}
+    } else if ((c == 'l') && (strncmp(LangString(args[2]), "list", length) == 0)) {
+        ListFactory listFactory;
+	TkTextLine *linePtr;
+	int offset, dummy;
+	Arg arg = NULL;
+
+	if (argc != 5) {
+	    Tcl_AppendResult(interp, "wrong # args: should be \"",
+		    LangString(args[0]), " block list index1 index2\"",          NULL);
+	    return TCL_ERROR;
+	}
+	if (TkTextGetIndex(interp, textPtr, LangString(args[3]), &index1) != TCL_OK
+	    || TkTextGetIndex(interp, textPtr, LangString(args[4]), &index2) != TCL_OK) {
+	    return TCL_ERROR;
+	}
+	if (TkTextIndexCmp(&index1, &index2) >= 0) {
+	    return TCL_OK;
+	}
+	linePtr = index1.linePtr;
+	offset = index1.charIndex; 
+	ListFactoryInit(&listFactory);
+	LangSetString(&arg, "");
+	while (linePtr != index2.linePtr) {
+	  TkBTreeLineTextAppend(textPtr, linePtr, offset, -1, 
+				(void*)&listFactory, 
+				textPtr->getFlags | TKT_GET_LIST, NULL);
+	  offset = 0;
+	  ListFactoryAppendCopyWithType(&listFactory, arg, 
+					IdType("NL",
+					       "Tk::Text::NewLine"));
+	  linePtr = TkBTreeNextLine(linePtr);
+	}
+	TkBTreeLineTextAppend(textPtr, linePtr, offset, index2.charIndex,
+			      (void*)&listFactory, 
+			      textPtr->getFlags | TKT_GET_LIST, NULL);
+	ListFactoryResult(textPtr->interp, &listFactory);
+	ListFactoryFree(&listFactory);
     } else if ((c == 's') && (strncmp(LangString(args[2]), "split", length) == 0)) {
         int level = 0;
 	char *end;
@@ -2293,7 +2373,7 @@ TkTextBlockCmd(textPtr, interp, argc, args)
     } else {
 	Tcl_AppendResult(interp, "bad block option \"", LangString(args[2]),
 		"\":  must be at, addline, cget, configure, delete, ",
-		"deletelines, insert, names, of, split or trim",
+		"deletelines, insert, list, names, of, split or trim",
 		         NULL);
 	return TCL_ERROR;
     }
