@@ -18,7 +18,7 @@
  *
  *
  * 
- * $Id: tkText.h,v 1.23 1995/09/09 06:37:48 ilya Exp ilya $
+ * $Id: tkText.h,v 1.27 1996/03/12 00:09:15 ilya Exp $
  *
  * 
  */
@@ -29,9 +29,11 @@
 #ifndef LangIsPerl
 #  include "ptcl.h"
 #  define TEXT_Class "Text"
+#  define IdType(id,type) id
 #else
 #  include "pperl.h"
 #  define TEXT_Class "eText"
+#  define IdType(id,type) type
 #endif 
 
 #ifndef _TK
@@ -375,6 +377,7 @@ typedef struct TkTextTag {
     int affectsDisplay;		/* Non-zero means that this tag affects the
 				 * way information is displayed on the screen
 				 * (so need to redisplay if tag changes). */
+    Arg userData;		/* arbitary user data */
 } TkTextTag;
 
 #define TK_TAG_AFFECTS_DISPLAY	0x1
@@ -656,6 +659,7 @@ typedef struct TkText {
 #define TKT_GET_OFFSET		0x4000
 #define TKT_GET_BUFFER		0x2000
 #define TKT_GET_DSTRING		0x1000
+#define TKT_GET_LIST		0x800
 
 /*
  * Records of the following type define segment types in terms of
@@ -932,9 +936,11 @@ extern void		TkTextIndexToExternalIndex _ANSI_ARGS_((
 			    TkTextBTree tree, TkTextIndex *indexPtr, 
 			    TkTextIndex *resIndexPtr));
 int			TkIBTreeTextAppend _ANSI_ARGS_((
+			    TkText *textPtr,
 			    TkTextSegment *segPtr, int start, int end,
 			    void *targetPtr, int flags, int* offsetPtr));
 int			TkBTreeLineTextAppend _ANSI_ARGS_((
+			    TkText *textPtr,
 			    TkTextLine *linePtr, int start, int end,
 			    void *targetPtr, int flags, int* offsetPtr));
 void			TkIBTreeFixTags _ANSI_ARGS_((
